//
// FLTK extension: type diversification of drag-n-drop
// Christoph Dalitz, 20.12.2004
//

#include <FL/Fl.H>
#include <FL/x.H>
#include <FL/Fl_Window.H>

// stuff from Fl_mac
extern EventRef fl_os_event;
extern char *fl_selection_buffer;
extern int fl_selection_length;

enum Fl_Dnd_Type { // data types for Drag-n-Drop
  FL_DND_TEXT       = 0,
  FL_DND_FILENAME   = 1
};

/**
 * drag and drop the data in the cut-copy-paste buffer
 * - create a selection first using: 
 *     Fl::copy(const char *stuff, int len, 0)
 */
int dnd_file(Fl_Dnd_Type ftype = FL_DND_TEXT)
{
  OSErr result;
  FSRef fref;
  HFSFlavor hflav;
  DragReference dragRef;

  if (ftype == FL_DND_FILENAME) {
    // create a file object (HFSFlavor) for drag-n-drop
    hflav.fileType=hflav.fileCreator=0; hflav.fdFlags=0;
    if (noErr != FSPathMakeRef((UInt8*)fl_selection_buffer, &fref, NULL))
      return false;
    if (noErr != FSGetCatalogInfo(&fref, kFSCatInfoNone, NULL, NULL,
                                 &(hflav.fileSpec), NULL))
      return false;

    // transfer fileobject to drop manager
    result = NewDrag( &dragRef );
    if ( result != noErr ) return false;
    result = AddDragItemFlavor( dragRef, 1, kDragFlavorTypeHFS, 
                                &hflav, sizeof(hflav), 0 );
    if ( result != noErr ) { DisposeDrag( dragRef ); return false; }
  }
  else {  //  (ftype == FL_DND_TEXT)
    result = NewDrag( &dragRef );
    if ( result != noErr ) return false;
    result = AddDragItemFlavor( dragRef, 1, 'TEXT', fl_selection_buffer,
                                fl_selection_length, 0 );
    if ( result != noErr ) { DisposeDrag( dragRef ); return false; }
  }

  // rest is the same as the old Fl::dnd()
  Point mp;
  GetMouse(&mp);
  LocalToGlobal( &mp );
  RgnHandle region = NewRgn();
  SetRectRgn( region, mp.h-10, mp.v-10, mp.h+10, mp.v+10 );
  RgnHandle r2 = NewRgn();
  SetRectRgn( r2, mp.h-8, mp.v-8, mp.h+8, mp.v+8 );
  DiffRgn( region, r2, region );
  DisposeRgn( r2 );

  EventRecord event;
  ConvertEventRefToEventRecord( fl_os_event, &event );
  result = TrackDrag( dragRef, &event, region );

  Fl_Widget *w = Fl::pushed();
  if ( w )
  {
    w->handle( FL_RELEASE );
    Fl::pushed( 0 );
  }

  if ( result != noErr ) { DisposeRgn( region ); DisposeDrag( dragRef ); return false; }
  
  DisposeRgn( region );
  DisposeDrag( dragRef );
  return true;
}
